@@ -1,5 +1,3 @@
-from __future__ import print_function, division, absolute_import
-
 import click
 from distributed.cli.utils import check_python_3, install_signal_handlers
 from distributed.submit import _remote
@@ -7,7 +5,9 @@ from distributed.submit import _remote
 
 @click.command()
 @click.option("--host", type=str, default=None, help="IP or hostname of this server")
-@click.option("--port", type=int, default=8788, help="Remote Client Port")
+@click.option(
+    "--port", type=int, default=8788, show_default=True, help="Remote Client Port"
+)
 @click.version_option()
 def main(host, port):
     _remote(host, port)
