@@ -1,6 +1,5 @@
 import atexit
 import logging
-import multiprocessing
 import gc
 import os
 from sys import exit
@@ -8,8 +7,10 @@ import warnings
 
 import click
 import dask
+from dask.utils import ignoring
 from distributed import Nanny, Worker
 from distributed.security import Security
+from distributed.system import CPU_COUNT
 from distributed.cli.utils import check_python_3, install_signal_handlers
 from distributed.comm import get_address_host_port
 from distributed.preloading import validate_preload_argv
@@ -315,7 +316,7 @@ def main(
         port = worker_port
 
     if not nthreads:
-        nthreads = multiprocessing.cpu_count() // nprocs
+        nthreads = CPU_COUNT // nprocs
 
     if pid_file:
         with open(pid_file, "w") as f:
@@ -354,6 +355,9 @@ def main(
             "dask-worker SCHEDULER_ADDRESS:8786"
         )
 
+    with ignoring(TypeError, ValueError):
+        name = int(name)
+
     nannies = [
         t(
             scheduler,
@@ -367,7 +371,9 @@ def main(
             port=port,
             dashboard_address=dashboard_address if dashboard else None,
             service_kwargs={"dashboard": {"prefix": dashboard_prefix}},
-            name=name if nprocs == 1 or not name else name + "-" + str(i),
+            name=name
+            if nprocs == 1 or name is None or name == ""
+            else str(name) + "-" + str(i),
             **kwargs
         )
         for i in range(nprocs)
