@@ -1,18 +1,23 @@
 import asyncio
 import atexit
 import copy
+import logging
 import math
 import weakref
 
+import dask
 from tornado import gen
 
 from .cluster import Cluster
 from ..core import rpc, CommClosedError
-from ..utils import LoopRunner, silence_logging, ignoring, parse_bytes
+from ..utils import LoopRunner, silence_logging, ignoring, parse_bytes, parse_timedelta
 from ..scheduler import Scheduler
 from ..security import Security
 
 
+logger = logging.getLogger(__name__)
+
+
 class ProcessInterface:
     """
     An interface for Scheduler and Worker processes for use in SpecCluster
@@ -22,7 +27,7 @@ class ProcessInterface:
     It should implement the methods below, like ``start`` and ``close``
     """
 
-    def __init__(self):
+    def __init__(self, scheduler=None, name=None):
         self.address = None
         self.external_address = None
         self.lock = asyncio.Lock()
@@ -160,6 +165,23 @@ class SpecCluster(Cluster):
     Also note that uniformity of the specification is not required.  Other API
     could be added externally (in subclasses) that adds workers of different
     specifications into the same dictionary.
+
+    If a single entry in the spec will generate multiple dask workers then
+    please provide a `"group"` element to the spec, that includes the suffixes
+    that will be added to each name (this should be handled by your worker
+    class).
+
+    >>> cluster.worker_spec
+    {
+        0: {"cls": MultiWorker, "options": {"processes": 3}, "group": ["-0", "-1", -2"]}
+        1: {"cls": MultiWorker, "options": {"processes": 2}, "group": ["-0", "-1"]}
+    }
+
+    These suffixes should correspond to the names used by the workers when
+    they deploy.
+
+    >>> [ws.name for ws in cluster.scheduler.workers.values()]
+    ["0-0", "0-1", "0-2", "1-0", "1-1"]
     """
 
     _instances = weakref.WeakSet()
@@ -184,6 +206,7 @@ class SpecCluster(Cluster):
         self._i = 0
         self.security = security or Security()
         self.scheduler_comm = None
+        self._futures = set()
 
         if silence_logs:
             self._old_logging_level = silence_logging(level=silence_logs)
@@ -250,13 +273,14 @@ class SpecCluster(Cluster):
             if to_close:
                 if self.scheduler.status == "running":
                     await self.scheduler_comm.retire_workers(workers=list(to_close))
-                tasks = [self.workers[w].close() for w in to_close]
+                tasks = [self.workers[w].close() for w in to_close if w in self.workers]
                 await asyncio.wait(tasks)
                 for task in tasks:  # for tornado gen.coroutine support
                     with ignoring(RuntimeError):
                         await task
             for name in to_close:
-                del self.workers[name]
+                if name in self.workers:
+                    del self.workers[name]
 
             to_open = set(self.worker_spec) - set(self.workers)
             workers = []
@@ -276,6 +300,22 @@ class SpecCluster(Cluster):
                     await w  # for tornado gen.coroutine support
             self.workers.update(dict(zip(to_open, workers)))
 
+    def _update_worker_status(self, op, msg):
+        if op == "remove":
+            name = self.scheduler_info["workers"][msg]["name"]
+
+            def f():
+                if name in self.workers and msg not in self.scheduler_info:
+                    self._futures.add(asyncio.ensure_future(self.workers[name].close()))
+                    del self.workers[name]
+
+            delay = parse_timedelta(
+                dask.config.get("distributed.deploy.lost-worker-timeout")
+            )
+
+            asyncio.get_event_loop().call_later(delay, f)
+        super()._update_worker_status(op, msg)
+
     def __await__(self):
         async def _():
             if self.status == "created":
@@ -288,18 +328,6 @@ class SpecCluster(Cluster):
 
         return _().__await__()
 
-    async def _wait_for_workers(self):
-        while {
-            str(d["name"])
-            for d in (await self.scheduler_comm.identity())["workers"].values()
-        } != set(map(str, self.workers)):
-            if (
-                any(w.status == "closed" for w in self.workers.values())
-                and self.scheduler.status == "running"
-            ):
-                raise gen.TimeoutError("Worker unexpectedly closed")
-            await asyncio.sleep(0.1)
-
     async def _close(self):
         while self.status == "closing":
             await asyncio.sleep(0.1)
@@ -309,13 +337,15 @@ class SpecCluster(Cluster):
 
         self.scale(0)
         await self._correct_state()
+        for future in self._futures:
+            await future
         async with self._lock:
             with ignoring(CommClosedError):
                 await self.scheduler_comm.close(close_workers=True)
 
         await self.scheduler.close()
         for w in self._created:
-            assert w.status == "closed"
+            assert w.status == "closed", w.status
 
         if hasattr(self, "_old_logging_level"):
             silence_logging(self._old_logging_level)
@@ -333,24 +363,32 @@ class SpecCluster(Cluster):
 
     def scale(self, n=0, memory=None, cores=None):
         if memory is not None:
-            try:
-                limit = self.new_spec["options"]["memory_limit"]
-            except KeyError:
+            for name in ["memory_limit", "memory"]:
+                try:
+                    limit = self.new_spec["options"][name]
+                except KeyError:
+                    pass
+                else:
+                    n = max(n, int(math.ceil(parse_bytes(memory) / parse_bytes(limit))))
+                    break
+            else:
                 raise ValueError(
                     "to use scale(memory=...) your worker definition must include a memory_limit definition"
                 )
-            else:
-                n = max(n, int(math.ceil(parse_bytes(memory) / parse_bytes(limit))))
 
         if cores is not None:
-            try:
-                threads_per_worker = self.new_spec["options"]["nthreads"]
-            except KeyError:
+            for name in ["nthreads", "ncores", "threads", "cores"]:
+                try:
+                    threads_per_worker = self.new_spec["options"][name]
+                except KeyError:
+                    pass
+                else:
+                    n = max(n, int(math.ceil(cores / threads_per_worker)))
+                    break
+            else:
                 raise ValueError(
                     "to use scale(cores=...) your worker definition must include an nthreads= definition"
                 )
-            else:
-                n = max(n, int(math.ceil(cores / threads_per_worker)))
 
         if len(self.worker_spec) > n:
             not_yet_launched = set(self.worker_spec) - {
@@ -367,8 +405,7 @@ class SpecCluster(Cluster):
             return
 
         while len(self.worker_spec) < n:
-            k, spec = self.new_worker_spec()
-            self.worker_spec[k] = spec
+            self.worker_spec.update(self.new_worker_spec())
 
         self.loop.add_callback(self._correct_state)
 
@@ -377,8 +414,7 @@ class SpecCluster(Cluster):
 
         Returns
         -------
-        name: identifier for worker
-        spec: dict
+        d: dict mapping names to worker specs
 
         See Also
         --------
@@ -387,13 +423,25 @@ class SpecCluster(Cluster):
         while self._i in self.worker_spec:
             self._i += 1
 
-        return self._i, self.new_spec
+        return {self._i: self.new_spec}
 
     @property
     def _supports_scaling(self):
         return not not self.new_spec
 
     async def scale_down(self, workers):
+        # We may have groups, if so, map worker addresses to job names
+        if not all(w in self.worker_spec for w in workers):
+            mapping = {}
+            for name, spec in self.worker_spec.items():
+                if "group" in spec:
+                    for suffix in spec["group"]:
+                        mapping[str(name) + suffix] = name
+                else:
+                    mapping[name] = name
+
+            workers = {mapping.get(w, w) for w in workers}
+
         for w in workers:
             if w in self.worker_spec:
                 del self.worker_spec[w]
@@ -401,12 +449,29 @@ class SpecCluster(Cluster):
 
     scale_up = scale  # backwards compatibility
 
-    def __repr__(self):
-        return "%s(%r, workers=%d)" % (
-            self._name,
-            self.scheduler_address,
-            len(self.workers),
-        )
+    @property
+    def plan(self):
+        out = set()
+        for name, spec in self.worker_spec.items():
+            if "group" in spec:
+                out.update({str(name) + suffix for suffix in spec["group"]})
+            else:
+                out.add(name)
+        return out
+
+    @property
+    def requested(self):
+        out = set()
+        for name in self.workers:
+            try:
+                spec = self.worker_spec[name]
+            except KeyError:
+                continue
+            if "group" in spec:
+                out.update({str(name) + suffix for suffix in spec["group"]})
+            else:
+                out.add(name)
+        return out
 
 
 @atexit.register
