@@ -1,8 +1,9 @@
 import asyncio
-from time import time
 
+import dask
 from dask.distributed import SpecCluster, Worker, Client, Scheduler, Nanny
 from distributed.deploy.spec import close_clusters, ProcessInterface
+from distributed.metrics import time
 from distributed.utils_test import loop, cleanup  # noqa: F401
 from distributed.utils import is_valid_xml
 import toolz
@@ -124,6 +125,37 @@ async def test_scale(cleanup):
         assert len(cluster.workers) == 1
 
 
+@pytest.mark.asyncio
+async def test_unexpected_closed_worker(cleanup):
+    worker = {"cls": Worker, "options": {"nthreads": 1}}
+    with dask.config.set({"distributed.deploy.lost-worker-timeout": "10ms"}):
+        async with SpecCluster(
+            asynchronous=True, scheduler=scheduler, worker=worker
+        ) as cluster:
+            assert not cluster.workers
+            assert not cluster.worker_spec
+
+            # Scale up
+            cluster.scale(2)
+            assert not cluster.workers
+            assert cluster.worker_spec
+
+            await cluster
+            assert len(cluster.workers) == 2
+
+            # Close one
+            await list(cluster.workers.values())[0].close()
+            start = time()
+            while len(cluster.workers) > 1:  # wait for messages to flow around
+                await asyncio.sleep(0.01)
+                assert time() < start + 2
+            assert len(cluster.workers) == 1
+            assert len(cluster.worker_spec) == 2
+
+            await cluster
+            assert len(cluster.workers) == 2
+
+
 @pytest.mark.asyncio
 async def test_broken_worker():
     with pytest.raises(Exception) as info:
@@ -152,7 +184,7 @@ async def test_new_worker_spec(cleanup):
     class MyCluster(SpecCluster):
         def new_worker_spec(self):
             i = len(self.worker_spec)
-            return i, {"cls": Worker, "options": {"nthreads": i + 1}}
+            return {i: {"cls": Worker, "options": {"nthreads": i + 1}}}
 
     async with MyCluster(asynchronous=True, scheduler=scheduler) as cluster:
         cluster.scale(3)
@@ -268,3 +300,101 @@ async def test_widget(cleanup):
 
         cluster.scale(5)
         assert "3 / 5" in cluster._widget_status()
+
+
+@pytest.mark.asyncio
+async def test_scale_cores_memory(cleanup):
+    async with SpecCluster(
+        scheduler=scheduler,
+        worker={"cls": Worker, "options": {"nthreads": 1}},
+        asynchronous=True,
+    ) as cluster:
+        cluster.scale(cores=2)
+        assert len(cluster.worker_spec) == 2
+        with pytest.raises(ValueError) as info:
+            cluster.scale(memory="5GB")
+
+        assert "memory" in str(info.value)
+
+
+@pytest.mark.asyncio
+async def test_ProcessInterfaceValid(cleanup):
+    async with SpecCluster(
+        scheduler=scheduler, worker={"cls": ProcessInterface}, asynchronous=True
+    ) as cluster:
+        cluster.scale(2)
+        await cluster
+        assert len(cluster.worker_spec) == len(cluster.workers) == 2
+
+        cluster.scale(1)
+        await cluster
+        assert len(cluster.worker_spec) == len(cluster.workers) == 1
+
+
+class MultiWorker(Worker, ProcessInterface):
+    def __init__(self, *args, n=1, name=None, nthreads=None, **kwargs):
+        self.workers = [
+            Worker(
+                *args, name=str(name) + "-" + str(i), nthreads=nthreads // n, **kwargs
+            )
+            for i in range(n)
+        ]
+
+    @property
+    def status(self):
+        return self.workers[0].status
+
+    def __str__(self):
+        return "<MultiWorker n=%d>" % len(self.workers)
+
+    __repr__ = __str__
+
+    async def start(self):
+        await asyncio.gather(*self.workers)
+
+    async def close(self):
+        await asyncio.gather(*[w.close() for w in self.workers])
+
+
+@pytest.mark.asyncio
+async def test_MultiWorker(cleanup):
+    async with SpecCluster(
+        scheduler=scheduler,
+        worker={
+            "cls": MultiWorker,
+            "options": {"n": 2, "nthreads": 4, "memory_limit": "4 GB"},
+            "group": ["-0", "-1"],
+        },
+        asynchronous=True,
+    ) as cluster:
+        s = cluster.scheduler
+        async with Client(cluster, asynchronous=True) as client:
+            cluster.scale(2)
+            await cluster
+            assert len(cluster.worker_spec) == 2
+            await client.wait_for_workers(4)
+
+            cluster.scale(1)
+            await cluster
+            assert len(s.workers) == 2
+
+            cluster.scale(memory="6GB")
+            await cluster
+            assert len(cluster.worker_spec) == 2
+            assert len(s.workers) == 4
+            assert cluster.plan == {ws.name for ws in s.workers.values()}
+
+            cluster.scale(cores=10)
+            await cluster
+            assert len(cluster.workers) == 3
+
+            adapt = cluster.adapt(minimum=0, maximum=4)
+
+            for i in range(adapt.wait_count):  # relax down to 0 workers
+                await adapt.adapt()
+            await cluster
+            assert not s.workers
+
+            future = client.submit(lambda x: x + 1, 10)
+            await future
+            assert len(cluster.workers) == 1
