@@ -6,6 +6,7 @@ import json
 import operator
 import sys
 from time import sleep
+import logging
 
 import dask
 from dask import delayed
@@ -23,6 +24,7 @@ from distributed.protocol.pickle import dumps
 from distributed.worker import dumps_function, dumps_task
 from distributed.utils import tmpfile
 from distributed.utils_test import (  # noqa: F401
+    captured_logger,
     cleanup,
     inc,
     dec,
@@ -1231,7 +1233,9 @@ def test_cancel_fire_and_forget(c, s, a, b):
     assert not s.tasks
 
 
-@gen_cluster(client=True, Worker=Nanny, clean_kwargs={"processes": False})
+@gen_cluster(
+    client=True, Worker=Nanny, clean_kwargs={"processes": False, "threads": False}
+)
 def test_log_tasks_during_restart(c, s, a, b):
     future = c.submit(sys.exit, 0)
     yield wait(future)
@@ -1258,6 +1262,15 @@ def test_reschedule(c, s, a, b):
     assert sum(future.key in a.data for future in x) <= 1
 
 
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
+def test_reschedule_warns(c, s, a, b):
+    with captured_logger(logging.getLogger("distributed.scheduler")) as sched:
+        s.reschedule(key="__this-key-does-not-exist__")
+
+    assert "not found on the scheduler" in sched.getvalue()
+    assert "Aborting reschedule" in sched.getvalue()
+
+
 @gen_cluster(client=True)
 def test_get_task_status(c, s, a, b):
     future = c.submit(inc, 1)
@@ -1614,3 +1627,17 @@ async def test_finished():
 
     await s.finished()
     await w.finished()
+
+
+@pytest.mark.asyncio
+async def test_retire_names_str(cleanup):
+    async with Scheduler(port=0) as s:
+        async with Worker(s.address, name="0") as a:
+            async with Worker(s.address, name="1") as b:
+                async with Client(s.address, asynchronous=True) as c:
+                    futures = c.map(inc, range(10))
+                    await wait(futures)
+                    assert a.data and b.data
+                    await s.retire_workers(names=[0])
+                    assert all(f.done() for f in futures)
+                    assert len(b.data) == 10
