@@ -25,7 +25,6 @@ import threading
 import warnings
 import weakref
 import pkgutil
-import six
 import tblib.pickling_support
 import xml.etree.ElementTree
 
@@ -334,7 +333,8 @@ def sync(loop, func, *args, callback_timeout=None, **kwargs):
         while not e.is_set():
             e.wait(10)
     if error[0]:
-        six.reraise(*error[0])
+        typ, exc, tb = error[0]
+        raise exc.with_traceback(tb)
     else:
         return result[0]
 
@@ -1434,7 +1434,9 @@ def format_dashboard_link(host, port):
         scheme = "https"
     else:
         scheme = "http"
-    return template.format(scheme=scheme, host=host, port=port, **os.environ)
+    return template.format(
+        **toolz.merge(os.environ, dict(scheme=scheme, host=host, port=port))
+    )
 
 
 def is_coroutine_function(f):
@@ -1453,12 +1455,13 @@ class Logs(dict):
 
     def _repr_html_(self):
         summaries = [
-            "<details>\n<summary>{title}</summary>\n{log}\n</details>".format(
-                title=title, log=log._repr_html_()
-            )
-            for title, log in self.items()
+            "<details>\n"
+            "<summary style='display:list-item'>{title}</summary>\n"
+            "{log}\n"
+            "</details>".format(title=title, log=log._repr_html_())
+            for title, log in sorted(self.items())
         ]
-        return "\n\n".join(summaries)
+        return "\n".join(summaries)
 
 
 def cli_keywords(d: dict, cls=None):
